summaryrefslogtreecommitdiffstats
path: root/apps
diff options
context:
space:
mode:
authorBjoern Schiessle <schiessle@owncloud.com>2014-04-16 16:41:23 +0200
committerBjoern Schiessle <schiessle@owncloud.com>2014-04-23 12:54:26 +0200
commit93469ca46865d02d33710a2f70f7f6092c8f5c58 (patch)
tree307247653ef5b55f95a00108377f037de6f02a31 /apps
parentdd1e47b3b896e2ee59faf5f423bb911c5d2c2548 (diff)
downloadnextcloud-server-93469ca46865d02d33710a2f70f7f6092c8f5c58.tar.gz
nextcloud-server-93469ca46865d02d33710a2f70f7f6092c8f5c58.zip
make it possible to move files out of a shared mount point
Diffstat (limited to 'apps')
-rw-r--r--apps/files_sharing/lib/helper.php31
-rw-r--r--apps/files_sharing/lib/sharedstorage.php27
2 files changed, 45 insertions, 13 deletions
diff --git a/apps/files_sharing/lib/helper.php b/apps/files_sharing/lib/helper.php
index e2780e98935..cc1f7d9ffdf 100644
--- a/apps/files_sharing/lib/helper.php
+++ b/apps/files_sharing/lib/helper.php
@@ -145,4 +145,35 @@ class Helper {
return $result;
}
+
+ public static function getUidAndFilename($filename) {
+ $uid = \OC\Files\Filesystem::getOwner($filename);
+ \OC\Files\Filesystem::initMountPoints($uid);
+ if ( $uid != \OCP\User::getUser() ) {
+ $info = \OC\Files\Filesystem::getFileInfo($filename);
+ $ownerView = new \OC\Files\View('/'.$uid.'/files');
+ $filename = $ownerView->getPath($info['fileid']);
+ }
+ return array($uid, $filename);
+ }
+
+ /**
+ * @brief Format a path to be relative to the /user/files/ directory
+ * @param string $path the absolute path
+ * @return string e.g. turns '/admin/files/test.txt' into 'test.txt'
+ */
+ public static function stripUserFilesPath($path) {
+ $trimmed = ltrim($path, '/');
+ $split = explode('/', $trimmed);
+
+ // it is not a file relative to data/user/files
+ if (count($split) < 3 || $split[1] !== 'files') {
+ return false;
+ }
+
+ $sliced = array_slice($split, 2);
+ $relPath = implode('/', $sliced);
+
+ return $relPath;
+ }
}
diff --git a/apps/files_sharing/lib/sharedstorage.php b/apps/files_sharing/lib/sharedstorage.php
index b8a799f720d..5e478d5ead8 100644
--- a/apps/files_sharing/lib/sharedstorage.php
+++ b/apps/files_sharing/lib/sharedstorage.php
@@ -362,6 +362,9 @@ class Shared extends \OC\Files\Storage\Common {
public function rename($path1, $path2) {
$sourceMountPoint = \OC\Files\Filesystem::getMountPoint($path1);
+ $targetMountPoint = \OC\Files\Filesystem::getMountPoint($path2);
+ $relPath1 = \OCA\Files_Sharing\Helper::stripUserFilesPath($path1);
+ $relPath2 = \OCA\Files_Sharing\Helper::stripUserFilesPath($path2);
// if we renamed the mount point we need to adjust the file_target in the
// database
@@ -369,21 +372,19 @@ class Shared extends \OC\Files\Storage\Common {
return $this->renameMountPoint($path1, $path2);
}
- // Renaming/moving is only allowed within shared folders
- $oldSource = $this->getSourcePath($path1);
- if ($oldSource) {
- $newSource = $this->getSourcePath(dirname($path2)) . '/' . basename($path2);
- // Within the same folder, we only need UPDATE permissions
- if (dirname($path1) == dirname($path2) and $this->isUpdatable($path1)) {
- list($storage, $oldInternalPath) = \OC\Files\Filesystem::resolvePath($oldSource);
- list(, $newInternalPath) = \OC\Files\Filesystem::resolvePath($newSource);
- return $storage->rename($oldInternalPath, $newInternalPath);
+
+ if ( // Within the same mount point, we only need UPDATE permissions
+ ($sourceMountPoint === $targetMountPoint && $this->isUpdatable($sourceMountPoint)) ||
// otherwise DELETE and CREATE permissions required
- } elseif ($this->isDeletable($path1) && $this->isCreatable(dirname($path2))) {
- $rootView = new \OC\Files\View('');
- return $rootView->rename($oldSource, $newSource);
- }
+ ($this->isDeletable($path1) && $this->isCreatable(dirname($path2)))) {
+
+ list($user1, $path1) = \OCA\Files_Sharing\Helper::getUidAndFilename($relPath1);
+ $targetFilename = basename($relPath2);
+ list($user2, $path2) = \OCA\Files_Sharing\Helper::getUidAndFilename(dirname($relPath2));
+ $rootView = new \OC\Files\View('');
+ return $rootView->rename('/' . $user1 . '/files/' . $path1, '/' . $user2 . '/files/' . $path2 . '/' . $targetFilename);
}
+
return false;
}